''' discussion_sync.py - sync discussion tickets located in Pagure'''

from blockerbugs import db, app
from blockerbugs.config import Config as bb_Config
from blockerbugs.util import pagure_interface
from blockerbugs.models.milestone import Milestone
from blockerbugs.models.bug import Bug


def needs_discussion(bug):
    return (bug.proposed_blocker
         or bug.proposed_fe
         or bug.rejected_blocker
         or bug.rejected_fe
         or bug.accepted_blocker
         or bug.accepted_0day
         or bug.accepted_prevrel
         or bug.accepted_fe)


def _link_key(milestone, bug):
    return "%d_%d" % (milestone.release.number, bug.bugid)


def create_discussions_links(milestone, bugs, links_to_reuse={}):
    app.logger.info('Creating discussion tickets for %d bugs under %s' % (
        len(bugs), milestone))

    for bug in bugs:
        if not needs_discussion(bug):
            app.logger.debug('Skipping bug %d, no discussion needed' % bug.bugid)
            continue

        link = links_to_reuse.get(_link_key(milestone, bug))
        if link:
            app.logger.debug('Reusing discussion link for bug %d' % bug.bugid)
        else:
            app.logger.debug('Creating Pagure discussion for bug %d' % bug.bugid)
            try:
                link = pagure_interface.create_bug_discussion(bug)
                links_to_reuse[_link_key(milestone, bug)] = link
            except pagure_interface.PagureAPIException as e:
                app.logger.error('Unable to create Pagure discussion for bug %s. '
                    'Pagure error: %s' % (bug.bugid, e))
                continue

        bug.discussion_link = link
        db.session.add(bug)
    db.session.commit()


def sync_discussions():
    if app.config["PAGURE_REPO_TOKEN"] in (bb_Config.PAGURE_REPO_TOKEN, ""):
        # skip this if the API token is set to the placeholder value
        # or empty string as we are not going to be able to create anything
        app.logger.info('Not syncing discussions, because PAGURE_REPO_TOKEN '
            'is not configured.')
        return

    links = {}

    active_milestones = Milestone.query.filter_by(active=True).all()

    for milestone in active_milestones:
        bugs = Bug.query.filter_by(milestone=milestone, active=True).all()
        for bug in (b for b in bugs if b.discussion_link):
            links.setdefault(_link_key(milestone, bug), bug.discussion_link)

    for milestone in active_milestones:
        bugs = Bug.query.filter_by(milestone=milestone, active=True, discussion_link=None).all()
        create_discussions_links(milestone, bugs, links_to_reuse=links)


def recreate_discussion(bugid):
    milestones = Milestone.query.all()
    for milestone in milestones:
        bugs = Bug.query.filter_by(bugid=bugid, milestone=milestone).all()
        create_discussions_links(milestone, bugs)
